@@ -96,10 +96,10 @@ assert_valid_tree_entry (RhythmDBTreeEntry *entry)
 #endif
 
 static RhythmDBEntry *rhythmdb_tree_entry_allocate (RhythmDBTree *db, RhythmDBEntryType type);
-static void rhythmdb_tree_entry_insert (RhythmDBTree *db, RhythmDBTreeEntry *entry,
-					RhythmDBEntryType type, const char *uri,
-					const char *genrename, const char *artistname,
-					const char *albumname);
+static gboolean rhythmdb_tree_entry_insert (RhythmDBTree *db, RhythmDBTreeEntry *entry,
+					    RhythmDBEntryType type, const char *uri,
+					    const char *genrename, const char *artistname,
+					    const char *albumname);
 
 static void destroy_tree_property (RhythmDBTreeProperty *prop);
 static RhythmDBTreeProperty *get_or_create_album (RhythmDBTree *db, RhythmDBTreeProperty *artist,
@@ -140,8 +140,6 @@ struct RhythmDBTreePrivate
 	GHashTable *song_genres;
 	GHashTable *iradio_genres;
 
-	GHashTable *propname_map;
-
 	gboolean finalizing;
 };
 
@@ -201,47 +199,9 @@ rhythmdb_tree_class_init (RhythmDBTreeClass *klass)
 	rhythmdb_class->impl_do_full_query = rhythmdb_tree_do_full_query;
 }
 
-static inline const char *
-elt_name_from_propid (RhythmDBPropType propid)
-{
-	switch (propid) {
-	case RHYTHMDB_PROP_TYPE:
-		return "type";
-	case RHYTHMDB_PROP_TITLE:
-		return "title";
-	case RHYTHMDB_PROP_GENRE:
-		return "genre";
-	case RHYTHMDB_PROP_ARTIST:
-		return "artist";
-	case RHYTHMDB_PROP_ALBUM:
-		return "album";
-	case RHYTHMDB_PROP_TRACK_NUMBER:
-		return "track-number";
-	case RHYTHMDB_PROP_DURATION:
-		return "duration";
-	case RHYTHMDB_PROP_FILE_SIZE:
-		return "file-size";
-	case RHYTHMDB_PROP_LOCATION:
-		return "location";
-	case RHYTHMDB_PROP_MTIME:
-		return "mtime";
-	case RHYTHMDB_PROP_RATING:
-		return "rating";
-	case RHYTHMDB_PROP_PLAY_COUNT:
-		return "play-count";
-	case RHYTHMDB_PROP_LAST_PLAYED:
-		return "last-played";
-	case RHYTHMDB_PROP_QUALITY:
-		return "quality";
-	}
-	g_assert_not_reached ();
-	return NULL;
-}
-
 static void
 rhythmdb_tree_init (RhythmDBTree *db)
 {
-	guint i;
 	db->priv = g_new0 (RhythmDBTreePrivate, 1);
 
 	db->priv->entries = g_hash_table_new (g_str_hash, g_str_equal);
@@ -260,23 +220,6 @@ rhythmdb_tree_init (RhythmDBTree *db)
 							 (GDestroyNotify) g_free,
 							 NULL);
 
-	db->priv->propname_map = g_hash_table_new (g_str_hash, g_str_equal);
-
-	for (i = 0; i < RHYTHMDB_NUM_SAVED_PROPERTIES; i++) {
-		const char *name = elt_name_from_propid (i);
-		g_hash_table_insert (db->priv->propname_map, (gpointer) name, GINT_TO_POINTER (i));
-	}
-}
-
-static inline int
-propid_from_elt_name (RhythmDBTree *db, const char *name)
-{
-	gpointer ret, orig;	
-	if (g_hash_table_lookup_extended (db->priv->propname_map, name,
-					  &orig, &ret)) {
-		return GPOINTER_TO_INT (ret);
-	}
-	return -1;
 }
 
 static inline char *
@@ -385,8 +328,6 @@ rhythmdb_tree_finalize (GObject *object)
 
 	g_hash_table_destroy (db->priv->entries);
 
-	g_hash_table_destroy (db->priv->propname_map);
-
 	g_mem_chunk_destroy (db->priv->entry_memchunk);
 	g_mem_chunk_destroy (db->priv->property_memchunk);
 
@@ -410,6 +351,7 @@ struct RhythmDBTreeLoadContext
 		RHYTHMDB_TREE_PARSER_STATE_ENTRY_PROPERTY,
 		RHYTHMDB_TREE_PARSER_STATE_END,
 	} state;
+	gboolean in_unknown_elt;
 	RhythmDBTreeEntry *entry;
 	char *genrename;
 	char *albumname;
@@ -427,18 +369,23 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 		return;
 	}
 
+	if (ctx->in_unknown_elt)
+		return;
+
 	switch (ctx->state)
 	{
 	case RHYTHMDB_TREE_PARSER_STATE_START:
 	{
 		if (!strcmp (name, "rhythmdb"))
 			ctx->state = RHYTHMDB_TREE_PARSER_STATE_RHYTHMDB;
+		else
+			ctx->in_unknown_elt = TRUE;
 		break;
 	}
 	case RHYTHMDB_TREE_PARSER_STATE_RHYTHMDB:
 	{
 		if (!strcmp (name, "entry")) {
-			RhythmDBEntryType type;
+			RhythmDBEntryType type = -1;
 			gboolean type_set = FALSE;
 			for (; *attrs; attrs +=2) {
 				if (!strcmp (*attrs, "type")) {
@@ -457,14 +404,17 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 			ctx->genrename = NULL;
 			ctx->albumname = NULL;
 			ctx->artistname = NULL;
-		}
+		} else
+			ctx->in_unknown_elt = TRUE;
 		break;
 	}
 	case RHYTHMDB_TREE_PARSER_STATE_ENTRY:
 	{
-		int val = propid_from_elt_name (ctx->db, name);
-		if (val < 0)
+		int val = rhythmdb_propid_from_nice_elt_name (RHYTHMDB (ctx->db), name);
+		if (val < 0) {
+			ctx->in_unknown_elt = TRUE;
 			break;
+		}
 		
 		ctx->state = RHYTHMDB_TREE_PARSER_STATE_ENTRY_PROPERTY;
 		ctx->propid = val;
@@ -484,6 +434,11 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 		xmlStopParser (ctx->xmlctx);
 		return;
 	}
+
+	if (ctx->in_unknown_elt) {
+		ctx->in_unknown_elt = FALSE;		
+		return;
+	}
 	
 	switch (ctx->state)
 	{
@@ -701,7 +656,7 @@ g_free (encoded);							\
 
 		value = RHYTHMDB_TREE_ENTRY_VALUE (entry, i);
 
-		elt_name = elt_name_from_propid (i);
+		elt_name = rhythmdb_nice_elt_name_from_propid ((RhythmDB *) ctx->db, i);
 
 		RHYTHMDB_FWRITE_STATICSTR ("    <", ctx->handle);
 		RHYTHMDB_FWRITE (elt_name, 1, strlen (elt_name), ctx->handle);
@@ -853,7 +808,7 @@ rhythmdb_tree_entry_allocate (RhythmDBTree *db, RhythmDBEntryType type)
 	return ret;
 }
 
-static void
+static gboolean
 rhythmdb_tree_entry_insert (RhythmDBTree *db, RhythmDBTreeEntry *entry,
 			    RhythmDBEntryType type,
 			    const char *uri,
@@ -865,7 +820,7 @@ rhythmdb_tree_entry_insert (RhythmDBTree *db, RhythmDBTreeEntry *entry,
 	char *new_uri;
 
 	if (g_hash_table_lookup (db->priv->entries, uri))
-		return;
+		return FALSE;
 
 	/* Initialize the tree structure. */
 	genre = get_or_create_genre (db, type, genrename);
@@ -877,6 +832,7 @@ rhythmdb_tree_entry_insert (RhythmDBTree *db, RhythmDBTreeEntry *entry,
 	g_hash_table_insert (db->priv->entries, new_uri, entry);
 	g_value_set_string_take_ownership (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_LOCATION),
 					   new_uri);
+	return TRUE;
 }
 
 
@@ -890,7 +846,10 @@ rhythmdb_tree_entry_new (RhythmDB *rdb, RhythmDBEntryType type, const char *uri)
 
 	ret = rhythmdb_tree_entry_allocate (db, type);
 
-	rhythmdb_tree_entry_insert (db, ret, type, uri, "", "", "");
+	if (!rhythmdb_tree_entry_insert (db, ret, type, uri, "", "", "")) {
+		rhythmdb_tree_entry_destroy (db, ret);
+		return NULL;
+	}
 
 	sanity_check_database (db);
 
@@ -933,6 +892,10 @@ get_or_create_genre (RhythmDBTree *db, RhythmDBEntryType type,
 	case RHYTHMDB_ENTRY_TYPE_IRADIO_STATION:
 		table = db->priv->iradio_genres;
 		break;
+	default:
+		g_assert_not_reached ();
+		table = NULL;
+		break;
 	}
 
 	genre = g_hash_table_lookup (table, name);		
@@ -1042,6 +1005,10 @@ remove_entry_from_album (RhythmDBTree *db, RhythmDBTreeEntry *entry)
 	case RHYTHMDB_ENTRY_TYPE_IRADIO_STATION:
 		table = db->priv->iradio_genres;
 		break;
+	default:
+		g_assert_not_reached ();
+		table = NULL;
+		break;
 	}
 
 	if (remove_child (RHYTHMDB_TREE_PROPERTY (entry->album), entry)) {
@@ -1379,6 +1346,7 @@ evaluate_conjunctive_subquery (RhythmDBTree *db, GPtrArray *query,
 		}
 		break;
 		case RHYTHMDB_QUERY_PROP_LIKE:
+		case RHYTHMDB_QUERY_PROP_NOT_LIKE:
 			if (G_VALUE_TYPE (data->val) == G_TYPE_STRING) {
 				gboolean islike;
 				const char *stra, *strb;
@@ -1400,7 +1368,11 @@ evaluate_conjunctive_subquery (RhythmDBTree *db, GPtrArray *query,
 
 				strb = g_value_get_string (data->val);
 				islike = (strstr (stra, strb) != NULL);
-				if (!islike)
+				if (data->type == RHYTHMDB_QUERY_PROP_LIKE
+				    && !islike)
+					return FALSE;
+				else if (data->type == RHYTHMDB_QUERY_PROP_NOT_LIKE
+				    && islike)
 					return FALSE;
 			} else {
 				if (rb_gvalue_compare (RHYTHMDB_TREE_ENTRY_VALUE (entry, data->propid),
