@@ -303,6 +303,7 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 						    FALSE, FALSE);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_TITLE);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_GENRE);
+	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_QUALITY);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_LAST_PLAYED);
@@ -414,10 +415,15 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	if (!entry)
 		entry = rhythmdb_entry_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, uri);
 	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_string (&val, title);
+	if (title)
+		g_value_set_string (&val, title);
+	else
+		g_value_set_string (&val, uri);
+
 	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_TITLE, &val);
+	g_value_reset (&val);
+		
 	if (genre) {
-		g_value_reset (&val);
 		g_value_set_string (&val, genre);
 		rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
 	}
@@ -444,8 +450,8 @@ impl_search (RBSource *asource, const char *search_text)
 	    source->priv->search_text != NULL
 	    && !strcmp (search_text, source->priv->search_text))
 		return;
-	if (search_text[0] == '\0')
-	  search_text = NULL;
+	if (search_text != NULL && search_text[0] == '\0')
+		search_text = NULL;
 
 	g_free (source->priv->search_text);
 	if (search_text)
@@ -453,6 +459,8 @@ impl_search (RBSource *asource, const char *search_text)
 	else
 		source->priv->search_text = NULL;
 	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_SEARCH);
+
+	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
 
 static RBEntryView *
@@ -633,6 +641,8 @@ genre_selected_cb (RBPropertyView *propview, const char *name,
 	g_free (iradio_source->priv->selected_genre);
 	iradio_source->priv->selected_genre = g_strdup (name);
 	rb_iradio_source_do_query (iradio_source, RB_IRADIO_QUERY_TYPE_GENRE);
+
+	rb_source_notify_filter_changed (RB_SOURCE (iradio_source));
 }
 
 static void
