@@ -414,6 +414,7 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ARTIST);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ALBUM);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_DURATION);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_QUALITY);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED);
@@ -637,6 +638,8 @@ genres_selected_cb (RBPropertyView *propview, GList *genres,
 	g_list_free (libsource->priv->selected_genres);
 	libsource->priv->selected_genres = genres;
 	rb_library_source_do_query (libsource, RB_LIBRARY_QUERY_TYPE_GENRE);
+
+	rb_source_notify_filter_changed (RB_SOURCE (libsource));
 }
 
 static void
@@ -660,6 +663,8 @@ artists_selected_cb (RBPropertyView *propview, GList *artists,
 	g_list_free (libsource->priv->selected_artists);
 	libsource->priv->selected_artists = artists;
 	rb_library_source_do_query (libsource, RB_LIBRARY_QUERY_TYPE_ARTIST);
+
+	rb_source_notify_filter_changed (RB_SOURCE (libsource));
 }
 
 static void
@@ -683,6 +688,8 @@ albums_selected_cb (RBPropertyView *propview, GList *albums,
 	g_list_free (libsource->priv->selected_albums);
 	libsource->priv->selected_albums = albums;
 	rb_library_source_do_query (libsource, RB_LIBRARY_QUERY_TYPE_ALBUM);
+
+	rb_source_notify_filter_changed (RB_SOURCE (libsource));
 }
 
 static void
@@ -744,6 +751,8 @@ impl_search (RBSource *asource, const char *search_text)
 	g_free (source->priv->search_text);
 	source->priv->search_text = search_text != NULL ? g_utf8_casefold (search_text, -1) : NULL;
 	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_SEARCH);
+
+	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
 
 
@@ -796,8 +805,10 @@ impl_reset_filters (RBSource *asource)
 	g_free (source->priv->search_text);
 	source->priv->search_text = NULL;
 
-	if (changed)
+	if (changed) {
 		rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_ALL);
+		rb_source_notify_filter_changed (RB_SOURCE (source));
+	}
 }
   
 static GtkWidget *
@@ -1224,6 +1235,7 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 	gboolean is_all_query, sorting_matches;
 	const char *current_sorting_type;
 
+	/* Unlocked */
 	rhythmdb_read_lock (source->priv->db);
 
 	if (source->priv->active_query) {
@@ -1348,8 +1360,6 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 	
 	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_MODEL (query_model));
 
-	rhythmdb_read_unlock (source->priv->db);
-
 	query = construct_query_from_selection (source);
 	
 	rhythmdb_do_full_query_async_parsed (source->priv->db, model, query);
