@@ -440,6 +440,7 @@ rb_playlist_source_set_query (RBPlaylistSource *source,
 
 	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_MODEL (query_model));
 
+	rhythmdb_read_lock (source->priv->db);
 	rhythmdb_do_full_query_async_parsed (source->priv->db, model, query);
 
 	g_object_unref (G_OBJECT (query_model));
@@ -695,8 +696,8 @@ playlist_iter_func (GtkTreeModel *model, GtkTreeIter *iter, char **uri, char **t
 
 	rhythmdb_read_lock (db);
 
-	*uri = (char*) rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_LOCATION);
-	*title = (char*) rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_TITLE);
+	*uri = g_strdup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_LOCATION));
+	*title = g_strdup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_TITLE));
 
 	rhythmdb_read_unlock (db);
 }
